//
//  Copyright (c) 2018 Touch Instinct
//
//  Permission is hereby granted, free of charge, to any person obtaining a copy
//  of this software and associated documentation files (the Software), to deal
//  in the Software without restriction, including without limitation the rights
//  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//  copies of the Software, and to permit persons to whom the Software is
//  furnished to do so, subject to the following conditions:
//
//  The above copyright notice and this permission notice shall be included in
//  all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//  THE SOFTWARE.
//

import UIKit.UIViewController

/// Base controller that should be configured with view model.
open class BaseConfigurableController<ViewModel>: UIViewController, ConfigurableController {

    /// Ability to set forced screen orientation
    open var forcedInterfaceOrientation: UIInterfaceOrientation?

    /// A view model instance used by this controller.
    public let viewModel: ViewModel

    open override var supportedInterfaceOrientations: UIInterfaceOrientationMask {
        switch forcedInterfaceOrientation {
        case .landscapeLeft:
            return .landscapeLeft

        case .landscapeRight:
            return .landscapeRight

        case .portrait:
            return .portrait

        case .portraitUpsideDown:
            return .portraitUpsideDown

        default:
            return super.supportedInterfaceOrientations
        }
    }

    open override var preferredInterfaceOrientationForPresentation: UIInterfaceOrientation {
        forcedInterfaceOrientation ?? super.preferredInterfaceOrientationForPresentation
    }

    /// Initializer with view model parameter.
    ///
    /// - Parameter viewModel: A view model to configure this controller.
    public init(viewModel: ViewModel) {
        self.viewModel = viewModel

        super.init(nibName: nil, bundle: nil)
    }

    required public init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }

    // MARK: - ConfigurableController

    open func addViews() {
        // override in subclass
    }

    open func configureLayout() {
        // override in subclass
    }

    open func bindViews() {
        // override in subclass
    }

    open func configureAppearance() {
        // override in subclass
    }

    open func localize() {
        // override in subclass
    }

    open func configureBarButtons() {
        // override in subclass
    }
}
